#include <iostream>
#include <complex>
#include <valarray>
#include <cassert>
#include <cmath>
#include <limits>

#define __MYDEBUG

#ifndef __DEBUG_H
#define __DEBUG_H

typedef double Double;
typedef std::complex<double> complex_double;
typedef std::complex<float> complex_float;
typedef std::complex<Double> Complex;


#ifndef NAN
        #define NAN numeric_limits<double>::signaling_NaN()
#endif

#ifndef isnan
    #define isnan(x) 0
#endif

#if ! HAVE_ISFINITE
    #define isfinite(x) (x < 1e30)  
#endif

const char* RightNow();

bool CheckArray( const std::valarray<Complex>& a, long l, char* s);
bool CheckIfArrayReal( const std::valarray<Complex>& a, long l, char* s);
bool CheckRealArray( const std::valarray<Double>& a, long l, char* s);
bool CheckValue( const Complex& a, long l, char* s);
bool CheckDoubleValue( const Double& a, long l, char* s);
bool CheckIfValidXForFFT( const std::valarray<Double>& a, long l, char* s);
bool CheckIfHermitian( const std::valarray<Double>& x, const std::valarray<Complex>& y, long l, char* s);

double ArgDiff(Complex a, Complex b);
double AbsDiff(Complex a, Complex b);

#ifdef __MYDEBUG
    #define MyAssert_(a)  if ( !(a) ) cerr << "Assertion failed at line " << __LINE__ << ", file "<<  __FILE__ << endl;
#else
    #define MyAssert_(a)  ;
#endif

#ifdef __MYDEBUG
    #define CheckValue_(a)          CheckValue(a, long(__LINE__), __FILE__)
    #define CheckDoubleValue_(a)    CheckDoubleValue(a, long(__LINE__), __FILE__)
    #define CheckArray_(a)          CheckArray(a, long(__LINE__), __FILE__)
    #define CheckRealArray_(a)      CheckRealArray(a, long(__LINE__), __FILE__)
    #define CheckIfArrayReal_(a)    CheckIfArrayReal(a, long(__LINE__), __FILE__)
    #define CheckIfValidXForFFT_(a) CheckIfValidXForFFT(a, long(__LINE__), __FILE__)
    #define CheckIfHermitian_(a,b)  CheckIfHermitian(a,b,long(__LINE__), __FILE__)
#else
    #define CheckValue_(a)          ;
    #define CheckDoubleValue_(a)    ;
    #define CheckArray_(a)          ;
    #define CheckRealArray_(a)      ;
    #define CheckIfArrayReal_(a)    ;
    #define CheckIfValidXForFFT_(a) ;
    #define CheckIfHermitian_(a,b)  ;
#endif

#endif
