#include "optics.h"
using namespace std;

/*
Everything is defined for Omega > 0
*/

 Complex w(	double inOmega, double inK, Complex e )
{
	Complex w = sqrt(SQR(inOmega/c)*e - SQR(inK));

	if (w.imag() < 0)
		w = conj(w);

	CheckValue_(w);
	return w;
}


 Complex index(Complex epsilon) 
{
	Complex n;
	n = sqrt(epsilon);

	if (n.imag() < 0)
		n = Complex(n.real(), -n.imag());

	CheckValue_(n);
	return n;
}



 Complex epsilon(	double n, 
					double k)
{
	Complex n_complex = Complex(fabs(n), fabs(k));
	
	return SQR(n_complex);

}

 Complex rpij(	double Omega, double K,
				Complex ei, Complex ej) 
{
	Complex v;
	Complex ni = index( ei);
	Complex nj = index(ej);
	Complex wi ;
	Complex wj;
	if (Omega == 0) {
		v = 0.;
	} else {
		wi = w(Omega, K, ei);
		wj = w(Omega, K, ej);
		v = (wi*ej-wj*ei)/(wi*ej+wj*ei);	
	}
	
	if (CheckValue_(v)) {
		cout << v << wi << wj;
	}
	return v;
}



 Complex rsij(	double Omega, double K,
				Complex ei, Complex ej ) 
{
	Complex v;
	Complex ni = index( ei);
	Complex nj = index( ej);
	Complex wi ;
	Complex wj;
	
	if (Omega == 0) {
		v = 0.;
	} else {
		wi = w(Omega, K, ei);
		wj = w(Omega, K, ej);
		v = (wi-wj)/(wi+wj);	
		
	}
	
	if (Omega < 0) {
		v = conj(v);
	}

	CheckValue_(v);

	return v;
}



 Complex tpij(	double Omega, double K,
				Complex ei, Complex ej) 
{
	Complex v;
	Complex ni = index( ei);
	Complex nj = index( ej);
	
	if (Omega == 0) {
		v = (double(2.) * ni * nj) / (ei + ej);
	} else {
		Complex wi = w(Omega, K, ei);
		Complex wj = w(Omega, K, ej);
		v = (ni*nj*wi)/(wi*ej+wj*ei);	
	}
	
	if (Omega < 0) {
		v = conj(v);
	}

	CheckValue_(v);
	return v;
}

 Complex tsij(	double Omega, double K,
				Complex ei, Complex ej) 
{
	Complex v;
	Complex ni = index( ei);
	Complex nj = index( ej);
	
	if (Omega == 0) {
		v = (double(2.) * ni) / (nj + ni);
	} else {
		Complex wi = w(Omega, K, ei);
		Complex wj = w(Omega, K, ej);
		v = (2.*wi)/(wi+wj);	
	}
	
	if (Omega < 0) {
		v = conj(v);
	}

	CheckValue_(v);
	return v;
}

