
#ifndef __THZPROP_H
#define __THZPROP_H

#include "cubicspline.h"

valarray<Complex> CalculateE_Omega_K_z_ForDiracInfiniteMedium ( 
                                const valarray<double>& inK,
                                double inOmega,
                                Complex inEpsilon1,
                                const valarray<Complex>& inPol_array);

valarray<Complex> CalculateE_Omega_K_z_ForThreeLayerGeometry (  
                                const valarray<double>& inK,
                                double inOmega,
                                double inNg,
                                double inAlpha,
                                double inAlpha_transverse,
                                double inD,
                                Complex inEpsilon1,
                                Complex inEpsilon2,
                                Complex inEpsilon3,
                                const valarray<Complex>& inPol_array,
                                double phaseMismatchFactor);

void
ApplyLensTransform(const valarray<double>& inK, valarray<Complex>& ioVx, double Omega,  Complex e1, double f, double L, double d_o, double d_i);

void
ApplyPropagationTransform(const valarray<double>& inK, valarray<Complex>& ioVx, double Omega,  Complex e1, double d_o);

Complex
IntegrateKdKover4pi2_WRT_arbK( const valarray<double>& inK,
                    const valarray<Complex>& inV,

                                        #ifdef HAVE_NUMERIC_LIMITS
                                            const double nu = numeric_limits<double>::infinity()
                                        #else
                                            const double nu = 1e100
                                        #endif
                                        );

void
InitPeakPolarizationFromFile(string filename, 
                         valarray<double>& ioOmega_array, 
                         valarray<Complex>& ioPeakPolarizationFromFile);
                         
valarray<Complex> PeakPolarizationFromFile( const valarray<double>&  inK,
        double inK_BW,
        long inOmegaIndex,
        valarray<Complex>& inPeakPolarizationFromFile);
        
valarray<Complex> PeakPolOneColourRect( const valarray<double>&  inK,
        double inK_BW,
        double inOmega,
        double inOmega_BW,
        double inTheoristsEwInCrystal,
        double inChi_eff);
        
valarray<Complex> PeakPolOneColourShift(    const valarray<double>&  inK,
        double inK_BW,
        double inOmega,
        double inOmega_BW,
        double inTheoristsE2wInCrystal,
        double inChi_eff,
        double inTau_eff,
        double inWp_eff);

valarray<Complex> PeakPolTwoColourInj(  const valarray<double>&  inK,
        double inK_BW,
        double inOmega,
        double inOmega_BW,
        double inTheoristsEwInCrystal,
        double inTheoristsE2wInCrystal,
        double inChi_eff,
        double inTau_eff,
        double inWp_eff);

Complex EpsilonAtOmegaForMaterial(double inOmega, unsigned long inMaterial);
Complex EOFilterAtOmega(double inOmega, Complex inEpsilon, double l, double ng_pump );

enum {kInvalid              = 1000,
    kGaAs                   = 1,
    kZnTe                   = 2,
    kGaP                    = 3, 
    kAir                    = 4, 
    kGaAsFrequencyDependent = 5,
    kZnTeFrequencyDependent = 6, 
    kGaPFrequencyDependent  = 7,
    kZnTeFrequencyDependentUpTo5THz = 8};
    
enum {kOneColourRectification=1, 
      kOneColourShift, 
      kOneColourInjection, 
      kTwoColourRectification, 
      kTwoColourShift, 
      kTwoColourInjection, 
      kPolarizationFromFile};
      
enum {kInfiniteMedium=1, kThreeLayerStructure};

Complex gEpsilon[kAir+1];
cubicspline gGaAs_n_FrequencyDependent;
cubicspline gZnTe_n_FrequencyDependent;
cubicspline gGaP_n_FrequencyDependent;
cubicspline gGaAs_k_FrequencyDependent;
cubicspline gZnTe_k_FrequencyDependent;
cubicspline gGaP_k_FrequencyDependent;

#define IN cin
#define OUT cout

#endif
